/*
 * Minecraft Forge
 * Copyright (c) 2016-2019.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version 2.1
 * of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package net.minecraftforge.client.model;

import com.google.common.collect.ImmutableList;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.client.renderer.vertex.VertexFormat;
import net.minecraft.client.renderer.vertex.VertexFormatElement;
import net.minecraft.client.renderer.vertex.VertexFormatElement.Type;
import net.minecraft.client.renderer.vertex.VertexFormatElement.Usage;

public class Attributes
{
    /*
     * Default format of the data in IBakedModel
     */
    public static final VertexFormat DEFAULT_BAKED_FORMAT = DefaultVertexFormats.field_176600_a;

    /*
     * Can first format be used where second is expected
     */
    public static boolean moreSpecific(VertexFormat first, VertexFormat second)
    {
        int size = first.func_177338_f();
        if(size != second.func_177338_f()) return false;

        int padding = 0;
        int j = 0;
        ImmutableList<VertexFormatElement> elementsFirst = first.func_227894_c_();
        ImmutableList<VertexFormatElement> elementsSecond = second.func_227894_c_();
        for(VertexFormatElement firstAttr : elementsFirst)
        {
            while(j < elementsSecond.size() && elementsSecond.get(j).func_177375_c() == Usage.PADDING)
            {
                padding += elementsSecond.get(j++).func_177368_f();
            }
            if(j >= elementsSecond.size() && padding == 0)
            {
                // if no padding is left, but there are still elements in first (we're processing one) - it doesn't fit
                return false;
            }
            if(padding == 0)
            {
                // no padding - attributes have to match
                VertexFormatElement secondAttr = elementsSecond.get(j++);
                if(
                    firstAttr.func_177369_e() != secondAttr.func_177369_e() ||
                    firstAttr.func_177370_d() != secondAttr.func_177370_d() ||
                    firstAttr.func_177367_b() != secondAttr.func_177367_b() ||
                    firstAttr.func_177375_c() != secondAttr.func_177375_c())
                {
                    return false;
                }
            }
            else
            {
                // padding - attribute should fit in it
                padding -= firstAttr.func_177368_f();
                if(padding < 0) return false;
            }
        }

        if(padding != 0 || j != elementsSecond.size()) return false;
        return true;
    }
}
